pub struct Place {
pub base_ty: StoredTy,
pub base: PlaceBase,
pub projections: Vec<Projection>,
}Expand description
A Place represents how a value is located in memory. This does not
always correspond to a syntactic place expression. For example, when
processing a pattern, a Place can be used to refer to the sub-value
currently being inspected.
Fields
base_ty: StoredTyThe type of the PlaceBase
base: PlaceBaseThe “outermost” place that holds this value.
projections: Vec<Projection>How this place is derived from the base place.
Implementations
Sourceimpl Place
impl Place
Sourcepub fn deref_tys<'db>(&self) -> impl Iterator<Item = Ty<'db>>
pub fn deref_tys<'db>(&self) -> impl Iterator<Item = Ty<'db>>
Returns an iterator of the types that have to be dereferenced to access
the Place.
The types are in the reverse order that they are applied. So if
x: &*const u32 and the Place is **x, then the types returned are
*const u32 then &*const u32.
Sourcepub fn ty<'db>(&self) -> Ty<'db>
pub fn ty<'db>(&self) -> Ty<'db>
Returns the type of this Place after all projections have been applied.
Sourcepub fn ty_before_projection<'db>(&self, projection_index: usize) -> Ty<'db>
pub fn ty_before_projection<'db>(&self, projection_index: usize) -> Ty<'db>
Returns the type of this Place immediately before projection_indexth projection
is applied.