pub(crate) struct PlaceWithOrigin {
pub origins: SmallVec<[CaptureSourceStack; 2]>,
pub place: Place,
}Expand description
A PlaceWithOrigin represents how a value is located in memory. This does not
always correspond to a syntactic place expression. For example, when
processing a pattern, a Place can be used to refer to the sub-value
currently being inspected.
Fields
origins: SmallVec<[CaptureSourceStack; 2]>ExprIds or PatIds of the expressions or patterns producing this value.
place: PlaceInformation about the Place.
Implementations
Sourceimpl PlaceWithOrigin
impl PlaceWithOrigin
fn new_no_projections<'db>( origin: impl Into<ExprOrPatIdPacked>, base_ty: Ty<'db>, base: PlaceBase, ) -> PlaceWithOrigin
fn new<'db>( origins: SmallVec<[CaptureSourceStack; 2]>, base_ty: Ty<'db>, base: PlaceBase, projections: Vec<Projection>, ) -> PlaceWithOrigin
fn push_projection(&mut self, projection: Projection, origin: ExprOrPatIdPacked)
pub(crate) fn span(&self) -> Span